package org.hardknockjava.lesson20.example02.reader.broadway;


/******************************************************************
 * This example uses JAXB to read an XML
 * file, dump the values obtained from it, and reproduce it
 * on the console.
 */

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.hardknockjava.lesson20.example02.jaxb.broadway.Broadway;
import org.hardknockjava.lesson20.example02.jaxb.broadway.Show;
import org.hardknockjava.lesson20.example02.jaxb.broadway.Show.Composers;
import org.hardknockjava.lesson20.example02.jaxb.broadway.Show.Lyricists;
import org.hardknockjava.lesson20.example02.jaxb.broadway.Show.Playwrights;

public class BroadwayReaderJAXB {
    private List<ShowPOJO> showList = new ArrayList<ShowPOJO>();
    private JAXBContext jaxbContext;
    private Unmarshaller jaxbUnmarshaller;
    private Marshaller jaxbMarshaller;

    public static void main(String[] args) throws Exception {
        BroadwayReaderJAXB instance = new BroadwayReaderJAXB();

        // Read the file and store it in instances of ShowPOJO.
        instance.read("data/broadway.xml");

        // Dump the instances to the console.
        instance.dump();

        // Write the instances to the console.
        System.out.println();
        System.out.println(instance.write());
    }

    public BroadwayReaderJAXB() throws Exception {
        jaxbContext = JAXBContext.newInstance(Broadway.class);
        jaxbUnmarshaller = jaxbContext.createUnmarshaller();
        jaxbMarshaller = jaxbContext.createMarshaller();
    }

    // Read the XML and store it.
    private void read(String fileName) throws Exception {
        File showFile = new File(fileName);
        InputStream xmlSource = new FileInputStream(showFile);
        Broadway broadway = (Broadway) jaxbUnmarshaller.unmarshal(xmlSource);

        for (Show show : broadway.getShow()) {
            String title = show.getTitle();
            String theatre = show.getTheatre();
            List<String> playwrights = show.getPlaywrights().getPlaywright();

            List<String> composers = new ArrayList<String>();
            if (show.getComposers() != null) {
                composers = show.getComposers().getComposer();
            }

            List<String> lyricists = new ArrayList<String>();
            if (show.getLyricists() != null) {
                lyricists = show.getLyricists().getLyricist();
            }

            Date opening = getNodeDateValue(show.getOpening());
            Date closing = getNodeDateValue(show.getClosing());
            Integer previews = show.getPreviews();
            Integer performances = show.getPerformances();
            ShowPOJO instance = new ShowPOJO(title, theatre, playwrights, composers, lyricists, opening, closing,
                    previews, performances);
            showList.add(instance);
        }
    }

    // This method is for elements that contain the string representation
    // of dates.
    private Date getNodeDateValue(String dateStr) {
        Date returnValue = null;

        if (dateStr != null) {
            SimpleDateFormat fmt = new SimpleDateFormat("MM/dd/yyyy");
            try {
                returnValue = fmt.parse(dateStr);
            } catch (Exception e) {
                returnValue = null;
            }
        }

        return returnValue;
    }

    // Dump the list of Shows to the console.
    private void dump() {
        for (ShowPOJO show : showList) {
            System.out.println(show.toString());
        }
    }

    // Return XML.
    private String write() throws Exception {
        Broadway broadwayOut = new Broadway();

        for (int index = showList.size() - 1; index >= 0; index--) {
            SimpleDateFormat fmt = new SimpleDateFormat("MM/dd/yyyy");

            ShowPOJO production = showList.get(index);
            Show showXml = new Show();

            if (production.getClosing() != null) {
                showXml.setClosing(fmt.format(production.getClosing()));
            }
            if (production.getOpening() != null) {
                showXml.setOpening(fmt.format(production.getOpening()));
            }

            if (!production.getComposerList().isEmpty()) {
                Composers composers = new Composers();
                composers.getComposer().addAll(production.getComposerList());
                showXml.setComposers(composers);
            }

            if (!production.getLyricistList().isEmpty()) {
                Lyricists lyricists = new Lyricists();
                lyricists.getLyricist().addAll(production.getLyricistList());
                showXml.setLyricists(lyricists);
            }
            
            Playwrights playwrights = new Playwrights();
            playwrights.getPlaywright().addAll(production.getPlaywrightList());
            showXml.setPlaywrights(playwrights);
            
            showXml.setPerformances(production.getPerformances());
            showXml.setPreviews(production.getPreviews());
            showXml.setTheatre(production.getTheatre());
            showXml.setTheatre(production.getTitle());
            broadwayOut.getShow().add(showXml);
        }

        OutputStream output = new ByteArrayOutputStream();
        
        // This statement ensures the output is in "pretty print" format.
        jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
        
        jaxbMarshaller.marshal(broadwayOut, output);
        return output.toString();
    }
}
